@@ -20,6 +20,7 @@ configuration file as a dictionary:
           "stale lock timeout": ...,
           "cache lifespan": ...,
           "write cache": ...,
+          "bounds": { ... },
           "allowed origin": ...,
           "jpeg options": ...,
           "png options": ...
@@ -45,6 +46,9 @@ configuration file as a dictionary:
   0 or omitted.
 - "write cache" is an optional boolean value to allow skipping cache write
   altogether. This is defined on a per-layer basis. Defaults to true if omitted.
+- "bounds" is an optional dictionary of six tile boundaries to limit the
+  rendered area: low (lowest zoom level), high (highest zoom level), north,
+  west, south, and east (all in degrees).
 - "allowed origin" is an optional string that shows up in the response HTTP
   header Access-Control-Allow-Origin, useful for when you need to provide
   javascript direct access to response data such as GeoJSON or pixel values.
@@ -73,6 +77,14 @@ Sample PNG creation options:
       "palette": "filename.act"
     }
 
+Sample bounds:
+
+    {
+        "low": 9, "high": 15,
+        "south": 37.749, "west": -122.358,
+        "north": 37.860, "east": -122.113
+    }
+
 Metatile represents a larger area to be rendered at one time. Metatiles are
 represented in the configuration file as a dictionary:
 
@@ -110,6 +122,11 @@ from urlparse import urljoin
 
 from Pixels import load_palette, apply_palette
 
+try:
+    from PIL import Image
+except ImportError:
+    import Image
+
 from ModestMaps.Core import Coordinate
 
 class Metatile:
@@ -191,6 +208,9 @@ class Layer:
           write_cache:
             Allow skipping cache write altogether, default true.
 
+          bounds:
+            Instance of Config.Bounds for limiting rendered tiles.
+          
           allowed_origin:
             Value for the Access-Control-Allow-Origin HTTP response header.
 
@@ -206,7 +226,7 @@ class Layer:
           preview_ext:
             Tile name extension for slippy map layer preview, default "png".
     """
-    def __init__(self, config, projection, metatile, stale_lock_timeout=15, cache_lifespan=None, write_cache=True, allowed_origin=None, preview_lat=37.80, preview_lon=-122.26, preview_zoom=10, preview_ext='png'):
+    def __init__(self, config, projection, metatile, stale_lock_timeout=15, cache_lifespan=None, write_cache=True, allowed_origin=None, preview_lat=37.80, preview_lon=-122.26, preview_zoom=10, preview_ext='png', bounds=None):
         self.provider = None
         self.config = config
         self.projection = projection
@@ -222,6 +242,8 @@ class Layer:
         self.preview_zoom = preview_zoom
         self.preview_ext = preview_ext
         
+        self.bounds = bounds
+        
         self.bitmap_palette = None
         self.jpeg_options = {}
         self.png_options = {}
@@ -251,6 +273,9 @@ class Layer:
             Perform metatile slicing here as well, if required, writing the
             full set of rendered tiles to cache as we go.
         """
+        if self.bounds and self.bounds.excludes(coord):
+            raise NoTileLeftBehind(Image.new('RGB', (256, 256), (0x99, 0x99, 0x99)))
+        
         srs = self.projection.srs
         xmin, ymin, xmax, ymax = self.envelope(coord)
         width, height = 256, 256
@@ -453,18 +478,20 @@ def _preview(layer):
     return """<!DOCTYPE html>
 <html>
 <head>
-	<title>TileStache Preview: %(layername)s</title>
-    <script src="http://code.modestmaps.com/0.13.2/modestmaps.min.js" type="text/javascript"></script>
+    <title>TileStache Preview: %(layername)s</title>
+    <script src="http://code.modestmaps.com/tilestache/modestmaps.min.js" type="text/javascript"></script>
+    <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=0;" />
 </head>
-<body>
+<body style="position: absolute; width: 100%%; height: 100%%;">
     <script type="text/javascript">
     <!--
     
         var template = '{Z}/{X}/{Y}.%(ext)s';
         var provider = new com.modestmaps.TemplatedMapProvider(template);
-        var map = new com.modestmaps.Map(document.body, provider);
+        var map = new MM.Map(document.body, provider);
         map.setCenterZoom(new com.modestmaps.Location(%(lat).6f, %(lon).6f), %(zoom)d);
-        map.draw();
+        // hashify it
+        new MM.Hash(map);
     
     //-->
     </script>
