@@ -8,7 +8,7 @@ designers and cartographers.
 
 Documentation available at http://tilestache.org/doc/
 """
-__version__ = '1.19.0'
+__version__ = '1.21.3'
 
 import re
 
@@ -34,7 +34,7 @@ import Config
 _previous_configs = {}
 
 # regular expression for PATH_INFO
-_pathinfo_pat = re.compile(r'^/?(?P<l>\w.+)/(?P<z>\d+)/(?P<x>\d+)/(?P<y>\d+)\.(?P<e>\w+)$')
+_pathinfo_pat = re.compile(r'^/?(?P<l>\w.+)/(?P<z>\d+)/(?P<x>-?\d+)/(?P<y>-?\d+)\.(?P<e>\w+)$')
 _preview_pat = re.compile(r'^/?(?P<l>\w.+)/preview\.html$')
 
 def getTile(layer, coord, extension, ignore_cached=False):
@@ -329,11 +329,12 @@ class WSGITileServer:
     def _response(self, start_response, code, content='', mimetype='text/plain', allowed_origin=''):
         """
         """
-        start_response(code, [
-            ('Content-Type', mimetype),
-            ('Content-Length', str(len(content))),
-            ('Access-Control-Allow-Origin', allowed_origin)
-        ])
+        headers = [('Content-Type', mimetype), ('Content-Length', str(len(content)))]
+        
+        if allowed_origin:
+            headers.append(('Access-Control-Allow-Origin', allowed_origin))
+        
+        start_response(code, headers)
         return [content]
 
 def modpythonHandler(request):
